import unittest

import atheris
import atheris_libprotobuf_mutator
from atheris import fuzz_test_lib
from google.protobuf import wrappers_pb2


@atheris.instrument_func
def simple_proto_comparison(msg):
  if msg.value == "abc":
    raise RuntimeError("Solved")


class AtherisLibprotobufMutatorTests(unittest.TestCase):

  def testSimpleProtoComparison(self):
    fuzz_test_lib.run_fuzztest(
        simple_proto_comparison,
        custom_setup=atheris_libprotobuf_mutator.Setup,
        setup_kwargs={"proto": wrappers_pb2.StringValue},
        expected_output=b"Solved",
        timeout=60)


if __name__ == "__main__":
  unittest.main()



# Copyright 2022 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Helpers for using libprotobuf-mutator bindings with Atheris."""

from typing import Any, Callable, List, Text

import atheris

from . import _mutator


def Setup(argv: List[Text],
          test_one_proto_input: Callable[[Any], Any],
          proto: Callable[..., Any],
          use_binary=False,
          **kwargs):
  """Wrapper for atheris.Setup() for fuzzing functions that take protos as input.

  It configures a custom mutator and crossover functions using
  libprotobuf-mutator.

  Args:
    argv: command line arguments.
    test_one_proto_input: function that will be called by the fuzzing engine and
      takes a proto as input.
    proto: the type of protobuf object taken by test_one_proto_input().
    use_binary: whether to use binary or text protos.
    **kwargs: additional arguments to pass to atheris.Setup().
  Returns:
    argv with any arguments consumed by Atheris removed.
  """

  def _CustomMutator(data: bytes, max_size: int, seed: int):
    msg = proto()
    res = _mutator.CustomProtoMutator(use_binary, data, max_size, seed, msg)
    return res

  def _CustomCrossOver(data1: bytes, data2: bytes, max_size: int, seed: int):
    msg1 = proto()
    msg2 = proto()
    return _mutator.CustomProtoCrossOver(use_binary, data1, data2, max_size,
                                         seed, msg1, msg2)

  @atheris.instrument_func
  def TestOneProtoInputImpl(data: bytes):
    """The entry point for our fuzzer.

    This is a callback that will be repeatedly invoked with different arguments
    after Fuzz() is called. It will invoke the user-provided function that
    expects a proto as argument.

    Args:
      data: Bytestring coming from the fuzzing engine.
    """
    msg = proto()
    msg = _mutator.LoadProtoInput(use_binary, data, msg)
    if msg:
      test_one_proto_input(msg)

  return atheris.Setup(
      argv,
      TestOneProtoInputImpl,
      custom_mutator=_CustomMutator,
      custom_crossover=_CustomCrossOver,
      **kwargs)



# Copyright 2022 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Example for using libprotobuf_mutator with Atheris."""

import atheris
import sys

import atheris_libprotobuf_mutator
from google.protobuf import wrappers_pb2


@atheris.instrument_func
def TestOneProtoInput(msg):
  if msg.value == 13371337:
    raise RuntimeError('Solved!')


if __name__ == '__main__':
  atheris_libprotobuf_mutator.Setup(
      sys.argv, TestOneProtoInput, proto=wrappers_pb2.Int64Value)
  atheris.Fuzz()



"""Setuptools for libprotobuf_mutator bindings for Atheris."""

import os
import shutil
import subprocess
import sys

import setuptools
from setuptools import setup
from distutils import spawn
from distutils.command import build
import setuptools.command.build_ext


class BuildExtCommand(setuptools.command.build_ext.build_ext):
  """Build C++ extensions and public protos with Bazel."""

  def python_bin_path_args(self):
    return ["--define", f"PYTHON_BIN_PATH='{sys.executable}'"]

  def env(self):
    ret = os.environ.copy()
    ret["PYTHON_BIN_PATH"] = sys.executable
    return ret

  def finalize_options(self):
    super().finalize_options()
    bazel = spawn.find_executable("bazel")
    if not bazel:
      raise RuntimeError(
          "Could not find 'bazel' binary. Please visit "
          "https://docs.bazel.build/versions/master/install.html for "
          "installation instruction.")
    self._bazel_cmd = [bazel]

  def run(self):
    if self.dry_run:
      return
    ext = self.extensions[0]
    ext_full_path = self.get_ext_fullpath(ext.name)
    subprocess.check_call(
        self._bazel_cmd + ["build"] + self.python_bin_path_args() +
        ["-c", "opt", "--cxxopt=-std=c++17", "//:_mutator.so"],
        # Bazel should be invoked in a directory containing bazel WORKSPACE
        # file, which is the root directory.
        cwd=os.path.dirname(os.path.realpath(__file__)),
        env=self.env())
    built_ext_path = "bazel-bin/_mutator.so"
    os.makedirs(os.path.dirname(ext_full_path), exist_ok=True)
    print("Copying extension %s -> %s" % (
        built_ext_path,
        ext_full_path,
    ))
    shutil.copyfile(built_ext_path, ext_full_path)


setup(
    name="atheris_libprotobuf_mutator",
    version="0.1.2",
    author="atheris@google.com",
    author_email="atheris@google.com",
    url="https://github.com/google/atheris/",
    description="libprotobuf-mutator bindings for Python using Atheris.",
    long_description=open("README.md", "r").read(),
    long_description_content_type="text/markdown",
    py_modules=[
        "atheris_libprotobuf_mutator.helpers",
        "atheris_libprotobuf_mutator.__init__"
    ],
    ext_modules=[
        setuptools.Extension(
            "atheris_libprotobuf_mutator._mutator", sources=[])
    ],
    cmdclass={
        "build_ext": BuildExtCommand,
    },
    zip_safe=False,
)
